# BambangShop Publisher App
Tutorial and Example for Advanced Programming 2024 - Faculty of Computer Science, Universitas Indonesia

---

## About this Project
In this repository, we have provided you a REST (REpresentational State Transfer) API project using Rocket web framework.

This project consists of four modules:
1.  `controller`: this module contains handler functions used to receive request and send responses.
    In Model-View-Controller (MVC) pattern, this is the Controller part.
2.  `model`: this module contains structs that serve as data containers.
    In MVC pattern, this is the Model part.
3.  `service`: this module contains structs with business logic methods.
    In MVC pattern, this is also the Model part.
4.  `repository`: this module contains structs that serve as databases and methods to access the databases.
    You can use methods of the struct to get list of objects, or operating an object (create, read, update, delete).

This repository provides a basic functionality that makes BambangShop work: ability to create, read, and delete `Product`s.
This repository already contains a functioning `Product` model, repository, service, and controllers that you can try right away.

As this is an Observer Design Pattern tutorial repository, you need to implement another feature: `Notification`.
This feature will notify creation, promotion, and deletion of a product, to external subscribers that are interested of a certain product type.
The subscribers are another Rocket instances, so the notification will be sent using HTTP POST request to each subscriber's `receive notification` address.

## API Documentations

You can download the Postman Collection JSON here: https://ristek.link/AdvProgWeek7Postman

After you download the Postman Collection, you can try the endpoints inside "BambangShop Publisher" folder.
This Postman collection also contains endpoints that you need to implement later on (the `Notification` feature).

Postman is an installable client that you can use to test web endpoints using HTTP request.
You can also make automated functional testing scripts for REST API projects using this client.
You can install Postman via this website: https://www.postman.com/downloads/

## How to Run in Development Environment
1.  Set up environment variables first by creating `.env` file.
    Here is the example of `.env` file:
    ```bash
    APP_INSTANCE_ROOT_URL="http://localhost:8000"
    ```
    Here are the details of each environment variable:
    | variable              | type   | description                                                |
    |-----------------------|--------|------------------------------------------------------------|
    | APP_INSTANCE_ROOT_URL | string | URL address where this publisher instance can be accessed. |
2.  Use `cargo run` to run this app.
    (You might want to use `cargo check` if you only need to verify your work without running the app.)

## Mandatory Checklists (Publisher)
-   [ ] Clone https://gitlab.com/ichlaffterlalu/bambangshop to a new repository.
-   **STAGE 1: Implement models and repositories**
    -   [ ] Commit: `Create Subscriber model struct.`
    -   [ ] Commit: `Create Notification model struct.`
    -   [ ] Commit: `Create Subscriber database and Subscriber repository struct skeleton.`
    -   [ ] Commit: `Implement add function in Subscriber repository.`
    -   [ ] Commit: `Implement list_all function in Subscriber repository.`
    -   [ ] Commit: `Implement delete function in Subscriber repository.`
    -   [ ] Write answers of your learning module's "Reflection Publisher-1" questions in this README.
-   **STAGE 2: Implement services and controllers**
    -   [ ] Commit: `Create Notification service struct skeleton.`
    -   [ ] Commit: `Implement subscribe function in Notification service.`
    -   [ ] Commit: `Implement subscribe function in Notification controller.`
    -   [ ] Commit: `Implement unsubscribe function in Notification service.`
    -   [ ] Commit: `Implement unsubscribe function in Notification controller.`
    -   [ ] Write answers of your learning module's "Reflection Publisher-2" questions in this README.
-   **STAGE 3: Implement notification mechanism**
    -   [ ] Commit: `Implement update method in Subscriber model to send notification HTTP requests.`
    -   [ ] Commit: `Implement notify function in Notification service to notify each Subscriber.`
    -   [ ] Commit: `Implement publish function in Program service and Program controller.`
    -   [ ] Commit: `Edit Product service methods to call notify after create/delete.`
    -   [ ] Write answers of your learning module's "Reflection Publisher-3" questions in this README.

## Your Reflections
This is the place for you to write reflections:

### Mandatory (Publisher) Reflections

#### Reflection Publisher-1
1. Dalam Observer Pattern, biasanya Subscriber didefinisikan sebagai interface (trait dalam Rust) untuk memungkinkan fleksibilitas dalam implementasi yang berbeda. Namun, dalam kasus BambangShop, kita hanya memiliki satu jenis subscriber dengan metode yang sama, sehingga tidak diperlukan trait.
2. tidak cukup karena pencarian subscriber berdasarkan url atau id tidak efisien – Dengan Vec, kita harus melakukan iterasi manual untuk mencari subscriber tertentu. DashMap<String, Subscriber> lebih cocok karena akses cepat (O(1) untuk pencarian dan penghapusan) dan juga mencegah duplikasi (karena url bisa menjadi key).
3. Singleton Pattern digunakan untuk memastikan hanya ada satu instance dari suatu objek dalam keseluruhan aplikasi. Namun, dalam kasus ini, DashMap tetap diperlukan karena singleton tidak secara otomatis menyediakan akses thread-safe ke data bersama.

#### Reflection Publisher-2
1. Baik, saya akan memberikan jawaban dalam paragraf singkat:
Pemisahan Service dan Repository dari Model dalam arsitektur MVC dilakukan untuk mencapai prinsip Separation of Concerns. Model murni hanya mewakili struktur data, Service menangani logika bisnis, sedangkan Repository mengelola akses data.
2.  Jika hanya menggunakan Model, kode akan menjadi sangat kompleks karena setiap model harus menangani representasi data, logika bisnis, dan interaksi penyimpanan sekaligus
3. Postman dapat digunakan untuk dokumentasi API yang lebih terorganisir sehingga dokumentasi endpoint dapat dilakukan dengan efisien pada project kali ini atau pada group project.

#### Reflection Publisher-3
1. Berdasarkan implementasi kode yang telah kita bahas, dalam tutorial ini digunakan Push Model dari Observer Pattern. Pada model ini, publisher (NotificationService) secara aktif mengirimkan data dan notifikasi langsung ke para subscriber.
2. jika menggunakan other variation conthnya  Pull Model, kelemahan utamanya adalah beban kerja akan dipindahkan ke subscriber. Setiap subscriber harus secara berkala melakukan polling atau pengecekan untuk mendapatkan informasi terbaru, yang mengakibatkan peningkatan kompleksitas sistem, konsumsi sumber daya yang lebih tinggi, dan latensi informasi.
3. Tanpa menggunakan multi-threading dalam proses notifikasi, program akan mengalami blocking pada setiap pengiriman notifikasi. Ini berarti setiap proses notifikasi akan menunggu hingga selesai sebelum melanjutkan ke subscriber berikutnya, yang dapat menyebabkan waktu respons yang lambat.
