use std::thread;
use bambangshop::{Result, compose_error_response};
use rocket::http::Status;
use crate::model::notification::Notification;
use crate::model::product::Product;
use crate::model::subscriber::Subscriber;
use crate::repository::subscriber::SubscriberRepository;

pub struct NotificationService;

impl NotificationService {
    pub fn subscribe(product_type: &str, subscriber: Subscriber) -> Result<Subscriber> {
        let product_type_str: String = product_type.to_uppercase();
        let product_type_str: &str = product_type_str.as_str();
        let subscriber_result: Subscriber = SubscriberRepository::add(product_type_str, subscriber);
        return Ok(subscriber_result);
    }
    pub fn unsubscribe(product_type: &str, url:&str) -> Result<Subscriber> {
        let product_type_str: String = product_type.to_uppercase();
        let product_type_str: &str = product_type_str.as_str();
        let subscriber_result: Option<Subscriber> = SubscriberRepository::delete(product_type_str, url);
        if subscriber_result.is_none() {
            return Err(compose_error_response(
                Status::NotFound, 
                "Subscriber not found".to_string()));
        }
        return Ok(subscriber_result.unwrap());
    }

    pub fn notify(&self, product_type: &str, status: &str, product: Product) {
        let mut payload: Notification = Notification {
            product_title: product.clone().title,
            product_type: String::from(product_type),
            product_url: product.clone().get_url(),
            subscriber_name: String::from(""),
            status: String::from(status),
        };
    
        let subscribers: Vec<Subscriber> = SubscriberRepository::list_all(product_type);
    
        for subscriber in subscribers {
            payload.subscriber_name = subscriber.clone().name;
            let payload_clone = payload.clone();
            thread::spawn(move || subscriber.update(payload_clone));
        }
    }
}